<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class WC_ChargeAfter_API
 * ChargeAfter Api
 */
class WC_ChargeAfter_API {
	/**
	 * Get Charge By ChargeId
	 *
	 * @param $charge_id
	 *
	 * @return mixed|null
	 */
	public static function api_get_charge_data( $charge_id ) {
		$url = self::get_api_prefix() . WC_ChargeAfter_Helper::get_url_suffix() . '/v2/post-sale/charges/' . (string) $charge_id;

		return self::api_generic_post_sale( 'wp_safe_remote_get', $url );
	}

	/**
	 * Get Api Prefix
	 *
	 * @return string
	 */
	private static function get_api_prefix() {
		return 'https://api';
	}

	/**
	 * Generic post request
	 *
	 * @param $http_callable
	 * @param $url
	 * @param string $data
	 *
	 * @return mixed|null
	 */
	public static function api_generic_post_sale( $http_callable, $url, $data = '' ) {
		$args = wp_parse_args(
			array(
				'body' => $data
			),
			self::get_default_request_args($url)
		);

		$request = $http_callable( $url, array_filter( $args, 'WC_ChargeAfter_Helper::filter_empty_string' ) );
		if ( is_wp_error( $request ) || WC_ChargeAfter_Helper::is_http_error_status( $request['response'] ) ) {
			return null;
		}

		$body = wp_remote_retrieve_body( $request );

		return json_decode( $body, true );
	}

	/**
	 * Settle Charge By ChargeId
	 *
	 * @param $charge_id
	 *
	 * @return mixed|null
	 */
	public static function api_settle_charge( $charge_id ) {
		return self::api_post_sale( $charge_id, 'settles' );
	}

	/**
	 * Post Sale
	 *
	 * @param $charge_id
	 * @param $action
	 * @param string $data
	 *
	 * @return mixed|null
	 */
	public static function api_post_sale( $charge_id, $action, $data = '' ) {
		$url = self::get_api_prefix() . WC_ChargeAfter_Helper::get_url_suffix() . '/v2/post-sale/charges/' . (string) $charge_id . '/' . $action;

		return self::api_generic_post_sale( 'wp_safe_remote_post', $url, $data );
	}

	/**
	 * Void Charge By ChargeId
	 *
	 * @param $charge_id
	 *
	 * @return mixed|null
	 */
	public static function api_void_charge( $charge_id ) {
		return self::api_post_sale( $charge_id, 'voids' );
	}

	/**
	 * Refund Charge By ChargeId
	 *
	 * @param $charge_id
	 * @param $charge_data
	 *
	 * @return mixed|null
	 */
	public static function api_refund_charge( $charge_id, $charge_data ) {
		$amount = floatval( $charge_data['totalAmount'] - $charge_data['refundedAmount'] );

		return self::api_post_sale( $charge_id, 'refunds', json_encode( array( 'amount' => $amount ) ) );

	}

	/**
	 * Refund Charge Amount By ChargeId
	 *
	 * @param $charge_id
	 * @param $amount
	 *
	 * @return mixed|null
	 */
	public static function api_refund_amount( $charge_id, $amount ) {
		return self::api_post_sale( $charge_id, 'refunds', json_encode( array( 'amount' => floatval( $amount ) ) ) );
	}

	/**
	 * Create Charge By CheckoutToken
	 *
	 * @param $token
	 *
	 * @return mixed|null
	 */
	public static function api_create_charge( $token ) {
		$url = self::get_api_prefix() . WC_ChargeAfter_Helper::get_url_suffix() . '/v2/payment/charges';

		return self::api_generic_post_sale( 'wp_safe_remote_post', $url, json_encode( array( 'confirmationToken' => $token ) ) );
	}

	/**
	 * Payment Partially Update
	 *
	 * @param $chargeId
	 * @param $order_id
	 *
	 * @return array|WP_Error
	 */
	public static function api_payment_partially_update( $chargeId, $order_id ) {
		$url = self::get_api_prefix() . WC_ChargeAfter_Helper::get_url_suffix() . '/v2/post-sale/charges/' . $chargeId;

		return self::api_generic_patch_sale( $url, json_encode( array( 'merchantOrderId' => (string) $order_id ) ) );
	}

	/**
	 * Generic patch request
	 *
	 * @param $url
	 * @param $body
	 *
	 * @return array|WP_Error
	 */
	public static function api_generic_patch_sale( $url, $body ) {
		$args = wp_parse_args(
			array(
				'method' => 'PATCH',
				'body'   => $body
			),
			self::get_default_request_args($url)
		);

		$request = wp_safe_remote_request( $url, $args );
		if ( is_wp_error( $request ) || WC_ChargeAfter_Helper::is_http_error_status( $request['response'] ) ) {
			return null;
		}

		$body = wp_remote_retrieve_body( $request );

		return json_decode( $body, true );
	}

	/**
	 * Create session
	 *
	 * @return string|null
	 */
	public static function api_create_session() {
		$url = self::get_api_prefix() . WC_ChargeAfter_Helper::get_url_suffix() . '/api/sessions';
		$args = wp_parse_args(
			array(
				'headers' => array(
					'Authorization' => 'Bearer ' . WC_ChargeAfter_Helper::get_public_key(),
					'Content-Type'  => 'application/json'
				),
				'method'  => 'POST',
				'body'    => json_encode([
					'requestInfo' => [
						'flowType' => 'Apply',
						'channel'  => 'e_commerce',
						'source'   => 'Api'
					]
				]),
			),
			self::get_default_request_args($url)
		);

		$request = wp_safe_remote_request( $url, $args );
		if ( is_wp_error( $request ) || WC_ChargeAfter_Helper::is_http_error_status( $request['response'] ) ) {
			return null;
		}

		$body = wp_remote_retrieve_body( $request );
		$body = json_decode( $body, true );

		return key_exists('id', $body) ? $body['id'] : null;
	}

	/**
	 * Get Merchant Id By Session Id
	 *
	 * @param $session_id
	 *
	 * @return mixed|null
	 */
	public static function api_get_merchant_id_by_session_id( $session_id ) {
		$url = self::get_api_prefix() . WC_ChargeAfter_Helper::get_url_suffix() . '/api/sessions/' . $session_id . '?projection=MerchantId';
		$args = wp_parse_args(
			array(
				'headers' => array(
					'Authorization' => 'Bearer ' . WC_ChargeAfter_Helper::get_public_key(),
					'Content-Type'  => 'application/json'
				),
				'method'  => 'GET',
			),
			self::get_default_request_args($url)
		);

		$request = wp_safe_remote_get( $url, $args );
		if ( is_wp_error( $request ) || WC_ChargeAfter_Helper::is_http_error_status( $request['response'] ) ) {
			return null;
		}

		$body = wp_remote_retrieve_body( $request );
		$body = json_decode($body, true);

		return key_exists('data', $body) && isset($body['data']['merchantId']) ? $body['data']['merchantId'] : null;
	}

	/**
	 * Get merchant settings
	 *
	 * @param $merchant_id
	 *
	 * @return array|null
	 */
	public static function api_get_merchant_settings( $merchant_id ) {
		$json = @file_get_contents(WC_ChargeAfter_Helper::get_merchant_settings_with_cdn($merchant_id));
		if (!empty($json)) {
			$json_settings = json_decode($json, true);
			if (empty($json_settings)) {
				$json_settings = json_decode(gzdecode($json), true);
			}

			return $json_settings;
		}

		return null;
	}

	private static function get_default_request_args( $url ) {
		return array(
			'headers' => array(
				'Authorization' => 'Bearer ' . WC_ChargeAfter_Helper::get_private_key(),
				'Content-Type'  => 'application/json'
			),
			'timeout' => apply_filters( 'http_request_timeout', 0, $url ),
		);
	}
}
