<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class WC_ChargeAfter_Helper
 */
class WC_ChargeAfter_Helper {
	/**
	 * Note messages
	 */
	const NEW_AUTH_NOTE = 'ChargeAfter Charge authorized (Charge ID: [UUID])';
	const VOID_AUTH_NOTE = 'ChargeAfter Charge voided (Charge ID: [UUID])';
	const SETTLE_NOTE = 'ChargeAfter Charge settled (Charge ID: [UUID])';
	const REFUND_NOTE = 'ChargeAfter refunded [Amount] [Currency] (Refund ID: [UUID])';
	const SETTLE_FAILED_NOTE = 'ChargeAfter error: unable to settle charge. Charge ID [UUID] is already voided';
	const GENERAL_FAILED_NOTE = 'ChargeAfter error: could not complete operation';
	const AMOUNT_FAILED_NOTE = "Unable to settle Charge. You can't settle a Charge for an amount different than authorized ([Amount] [Currency]).";
	const REFUNDED_FAILED_NOTE = 'ChargeAfter error: unsettled payment cannot be refunded. Confirm or cancel the charge';
	const ADD_FEE_NOTE = 'ChargeAfter: added an additional fee [Name] on [Amount] [Currency]';

	/**
	 * Allow country
	 *
	 * @var array
	 */
	private static $allow_country = array( 'US' );

	/**
	 * Allow currency
	 *
	 * @var array
	 */
	private static $allow_currency = array( 'USD' );

	/**
	 * Check http error status
	 *
	 * @param $response
	 *
	 * @return bool
	 */
	public static function is_http_error_status( $response ) {
		$response_code = wp_remote_retrieve_response_code( $response );
		if ( is_int( $response_code ) && $response_code >= 400 && $response_code < 600 ) {
			return true;
		}

		return false;
	}

	/**
	 * Filter empty string
	 *
	 * @param $var
	 *
	 * @return bool
	 */
	public static function filter_empty_string( $var ) {
		if ( is_string( $var ) ) {
			return ( '' !== $var );
		} else if ( is_array( $var ) ) {
			return count( $var ) > 0;
		}

		return true;
	}

	/**
	 * Recursive sanitation for an array
	 *
	 * @param $array
	 * @param $key
	 *
	 * @return mixed
	 */
	public static function recursive_sanitize_text_field( $array, $key = null ) {
		if ( !empty($key) ) {
			$array[$key] = $array;
		}

		foreach ( $array as $key => &$value ) {
			if ( is_array( $value ) ) {
				$value = self::recursive_sanitize_text_field($value);
			} else {
				$value = sanitize_text_field( $value );
			}
		}

		return $array;
	}

	/**
	 * Get Url Suffix
	 *
	 * @return string
	 */
	public static function get_url_suffix() {
		return self::is_sandbox() ? '-sandbox.ca-dev.co' : '.chargeafter.com';
	}

	/**
	 * Check sandbox flag
	 *
	 * @return bool
	 */
	public static function is_sandbox() {
		return 'yes' !== self::get_setting_param( 'is_production' );
	}

	/**
	 * Get setting param
	 *
	 * @param $key
	 *
	 * @param null $default
	 *
	 * @return mixed|null
	 */
	public static function get_setting_param( $key, $default = null ) {
		$settings = self::get_all_settings();

		return key_exists( $key, $settings ) ? $settings[ $key ] : $default;
	}

	/**
	 * Get All settings
	 *
	 * @return mixed|void
	 */
	public static function get_all_settings() {
		return get_option( 'woocommerce_' . CHARGEAFTER_ID . '_settings', array() );
	}

	/**
	 * Get Merchant Id Key By Settings
	 *
	 * @return string
	 */
	public static function get_setting_merchant_id_key() {
		return 'woocommerce_' . CHARGEAFTER_ID . '_m_id';
	}

	/**
	 * Get Merchant PK Key By Settings
	 *
	 * @return string
	 */
	public static function get_setting_merchant_pk_key() {
		return 'woocommerce_' . CHARGEAFTER_ID . '_pk_id';
	}

	/**
	 * Check ChargeAfter Gateway for order
	 *
	 * @param $order_id
	 *
	 * @return bool
	 */
	public static function is_chargeafter_gateway( $order_id ) {
		$order = wc_get_order( $order_id );

		return ! empty( $order ) && ( CHARGEAFTER_ID == $order->get_payment_method() );
	}

	/**
	 * Check promo enabled
	 *
	 * @return bool
	 */
	public static function is_enabled_promo() {
		return 'yes' == self::get_setting_param( 'enabled_promo' );
	}

	/**
	 * Get promo type
	 *
	 * @return mixed|null
	 */
	public static function get_promo_type() {
		return self::get_setting_param( 'promo_type', 'product-widget-line-of-credit' );
	}

	/**
	 * Get promo url
	 *
	 * @return mixed|null
	 */
	public static function get_promo_url() {
		return self::get_setting_param( 'promo_url', '/' );
	}

	/**
	 * Allow gateway
	 *
	 * @return bool
	 */
	public static function is_allow_gateway() {
		return ( self::is_enabled() && ! empty( self::get_public_key() ) && ! empty( self::get_private_key() ) ) || is_admin();
	}

	/**
	 * Check enabled flag
	 *
	 * @return bool
	 */
	public static function is_enabled() {
		return 'yes' == self::get_setting_param( 'enabled' );
	}

	/**
	 * Get public key
	 *
	 * @return mixed
	 */
	public static function get_public_key() {
		return self::is_sandbox() ? self::get_setting_param( 'sandbox_public_key' )
			: self::get_setting_param( 'production_public_key' );
	}

	/**
	 * Get private key
	 *
	 * @return mixed
	 */
	public static function get_private_key() {
		return self::is_sandbox() ? self::get_setting_param( 'sandbox_private_key' )
			: self::get_setting_param( 'production_private_key' );
	}

	/**
	 * Is allow currency
	 *
	 * @return bool
	 */
	public static function is_allow_currency() {
		$currency = apply_filters( 'woocommerce_currency', get_option( 'woocommerce_currency' ) );

		return in_array( $currency, self::get_allow_currency() );
	}

	/**
	 * Allow currency
	 *
	 * @return array
	 */
	public static function get_allow_currency() {
		return self::$allow_currency;
	}

	/**
	 * Allow country
	 *
	 * @return array
	 */
	public static function get_allow_country() {
		return self::$allow_country;
	}

	/**
	 * Get checkout description
	 *
	 * @return string|void
	 */
	public static function get_checkout_description() {
		return ( self::get_checkout_brand() == 'synchrony_pay' )
					? __( 'Buy now, pay over time with special financing.', 'chargeafter-extension' )
					: __( 'Pay over time at your own pace. Up to 6 months promotional financing.', 'chargeafter-extension' );
	}

	/**
	 * Get checkout brand
	 *
	 * @return mixed|string
	 */
	public static function get_checkout_brand() {
		return self::checkout_brand_deprecated(self::get_setting_param('icon', 'chargeafter'));
	}

	/**
	 * Get checkout brand cdn
	 *
	 * @param $checkout_brand
	 *
	 * @return string
	 */
	public static function get_checkout_brand_with_cdn( $checkout_brand ) {
		$checkout_brand = self::checkout_brand_deprecated($checkout_brand);
		return self::get_cdn_url() . "assets/brands/{$checkout_brand}/button.svg";
	}

	/**
	 * Get brand id by merchant
	 *
	 * @param null $empty_value
	 * @return mixed|null
	 */
	public static function get_merchant_brand_id( $empty_value = null ) {
		if ( empty( self::get_public_key() ) ) {
			return $empty_value;
		}

		$merchant_id_key = self::get_setting_merchant_id_key();
		$merchant_pb_key  = self::get_setting_merchant_pk_key();

		$merchant_id  = get_option($merchant_id_key);
		$merchant_pb  = get_option($merchant_pb_key);

		if ( empty($merchant_id) || empty($merchant_pb) || self::get_public_key() != $merchant_pb ) {
			$session_id = WC_ChargeAfter_API::api_create_session();
			if ( empty( $session_id ) ) {
				return $empty_value;
			}

			$merchant_id = WC_ChargeAfter_API::api_get_merchant_id_by_session_id($session_id);
			if ( empty($merchant_id) ) {
				return $empty_value;
			}

			add_option($merchant_id_key, $merchant_id);
			add_option($merchant_pb_key, self::get_public_key());
		}

		$merchant_settings = WC_ChargeAfter_API::api_get_merchant_settings($merchant_id);
		if ( empty($merchant_settings) || !key_exists('brandId', $merchant_settings) ) {
			return $empty_value;
		}

		return mb_strtolower($merchant_settings['brandId']);
	}

	/**
	 * Get CDN url
	 *
	 * @return string
	 */
	public static function get_cdn_url() {
		return self::is_sandbox()
				? 'https://cdn-sandbox.ca-dev.co/'
				: 'https://storage.googleapis.com/cdn-production-bucket/';
	}

	/**
	 * Get CDN settings for merchant
	 *
	 * @param $merchant_id
	 *
	 * @return string
	 */
	public static function get_merchant_settings_with_cdn( $merchant_id ) {
		return self::get_cdn_url() . 'assets/merchants/' . $merchant_id . '/settings.json';
	}

	/**
	 * Old brand value deprecate
	 *
	 * @param $checkout_brand
	 *
	 * @return mixed|string
	 */
	public static function checkout_brand_deprecated( $checkout_brand ) {
		$oldBrandValue = [
			'Btn_CA' => 'chargeafter',
			'Btn_CF' => 'checkout_finance',
			'Btn_SatisFi' => 'satisfi',
			'Btn_Synchrony' => 'synchrony_pay'
		];

		return key_exists($checkout_brand, $oldBrandValue) ? $oldBrandValue[$checkout_brand] : $checkout_brand;
	}

	/**
	 * Check OnDataUpdate enabled
	 *
	 * @return bool
	 */
	public static function is_enabled_data_update() {
		return 'yes' == self::get_setting_param( 'enabled_data_update' );
	}

	/**
	 * Check Auto-settle enabled
	 *
	 * @return bool
	 */
	public static function is_enabled_auto_settle() {
		return 'sale' == self::get_setting_param( 'payment_action' , '' );
	}

	/**
	 * Add settle note
	 *
	 * @param $order_id
	 */
	public static function add_settle_note( $order_id ) {
		$order          = wc_get_order( $order_id );
		$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
		$order->add_order_note( str_replace( '[UUID]', $transaction_id, self::SETTLE_NOTE ) );
	}

	/**
	 * Add settleFailed note
	 *
	 * @param $order_id
	 */
	public static function add_settle_failed_note( $order_id ) {
		$order          = wc_get_order( $order_id );
		$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
		$order->add_order_note( str_replace( '[UUID]', $transaction_id, self::SETTLE_FAILED_NOTE ) );
	}

	/**
	 * Add voidAuth note
	 *
	 * @param $order_id
	 */
	public static function add_void_auth_note( $order_id ) {
		$order          = wc_get_order( $order_id );
		$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
		$order->add_order_note( str_replace( '[UUID]', $transaction_id, self::VOID_AUTH_NOTE ) );
	}

	/**
	 * Add refund note
	 *
	 * @param $order_id
	 * @param $amount
	 * @param $refund_transaction_id
	 */
	public static function add_refund_note( $order_id, $amount, $refund_transaction_id ) {
		$order = wc_get_order( $order_id );
		$note  = str_replace( '[UUID]', $refund_transaction_id, self::REFUND_NOTE );
		$note  = str_replace( '[Amount]', $amount, $note );
		$note  = str_replace( '[Currency]', $order->get_currency(), $note );
		$order->add_order_note( $note );
	}

	/**
	 * Add newAuth note
	 *
	 * @param $order_id
	 */
	public static function add_new_auth_note( $order_id ) {
		$order          = wc_get_order( $order_id );
		$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
		$order->add_order_note( str_replace( '[UUID]', $transaction_id, self::NEW_AUTH_NOTE ) );
	}

	/**
	 * Add generalFailed note
	 *
	 * @param $order_id
	 */
	public static function add_general_failed_note( $order_id ) {
		$order = wc_get_order( $order_id );
		$order->add_order_note( self::GENERAL_FAILED_NOTE );
	}

	/**
	 * Add amountFailed note
	 *
	 * @param $order_id
	 * @param $amount
	 */
	public static function add_amount_failed_note( $order_id, $amount ) {
		$order = wc_get_order( $order_id );
		$note  = str_replace( '[Amount]', $amount, self::AMOUNT_FAILED_NOTE );
		$note  = str_replace( '[Currency]', $order->get_currency(), $note );
		$order->add_order_note( $note );
	}

	/**
	 * Add refundedFailed note
	 *
	 * @param $order_id
	 */
	public static function add_refund_failed_note( $order_id ) {
		$order = wc_get_order( $order_id );
		$order->add_order_note( self::REFUNDED_FAILED_NOTE );
	}

    /**
     * Add new fee note
     *
     * @param $order_id
     * @param $fee_name
     * @param $fee_amount
     */
	public static function add_new_fee_note( $order_id, $fee_name, $fee_amount ) {
		$order = wc_get_order( $order_id );

        $note = str_replace( '[Name]', $fee_name, self::ADD_FEE_NOTE );
        $note = str_replace( '[Amount]', wc_format_decimal( $fee_amount ), $note );
        $note = str_replace( '[Currency]', $order->get_currency(), $note );

        $order->add_order_note( $note );
	}
}
