<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class WC_ChargeAfter_Order_Handler
 */
class WC_ChargeAfter_Order_Handler extends WC_Payment_Gateway {
	/**
	 * Class Instance
	 *
	 * @var WC_ChargeAfter_Order_Handler
	 */
	private static $_this;

	/**
	 * WC_ChargeAfter_Order_Handler constructor.
	 */
	public function __construct() {
		self::$_this = $this;

		/*
		 * Actions
		 */
		add_action( 'woocommerce_order_status_on-hold_to_completed', array( $this, 'order_on_hold_to_processing_or_completed' ) );
		add_action( 'woocommerce_order_status_on-hold_to_cancelled', array( $this, 'order_on_hold_to_cancelled' ) );
		add_action( 'woocommerce_order_status_completed_to_cancelled', array( $this, 'order_processing_or_completed_to_cancelled' ) );

		add_action( 'woocommerce_order_status_changed', array( $this, 'order_status_changed' ), 10, 4 );
		add_action( 'woocommerce_order_status_refunded', array( $this, 'order_status_refunded' ), 9 );
	}

	/**
	 * Get instance
	 *
	 * @return WC_ChargeAfter_Order_Handler
	 */
	public static function get_instance() {
		return self::$_this;
	}

	/**
	 * Hold to Processing OR Completed
	 *
	 * @param $order_id
	 */
	public function order_on_hold_to_processing_or_completed( $order_id ) {
		if ( WC_ChargeAfter_Helper::is_chargeafter_gateway( $order_id ) ) {

			$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
			$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

			if ( $charge_data ) {
				switch ( $charge_data['state'] ) {
					case WC_ChargeAfter_ChargeState::AUTHORIZED:
					case WC_ChargeAfter_ChargeState::PARTIALLY_SETTLED:
					case WC_ChargeAfter_ChargeState::CHARGEBACK:
						$this->try_settle_order( $order_id );
						break;

					case WC_ChargeAfter_ChargeState::VOIDED:
						WC_ChargeAfter_Helper::add_settle_failed_note( $order_id );

						$order = wc_get_order( $order_id );
						$order->update_status( 'failed', __( 'Failed payment', 'chargeafter-extension' ) );

						break;
				}
			}
		}
	}

	/**
	 * Try settle order
	 *
	 * @param $order_id
	 */
	private function try_settle_order( $order_id ) {
		$order          = wc_get_order( $order_id );
		$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
		$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

		if ( $charge_data ) {
			$charge_amount = $charge_data['totalAmount'];

			if ($charge_amount < $order->get_total()) {
				$this->try_add_order_fee( $order_id, $charge_amount - $order->get_total() );
				$order = wc_get_order( $order_id );
			}

            if ( $order->get_total() == $charge_amount ) {
				if ( WC_ChargeAfter_API::api_settle_charge( $transaction_id ) ) {
					WC_ChargeAfter_Helper::add_settle_note( $order_id );
				} else {
					$this->settle_failed_indication( $order_id );
				}
			} else {
				WC_ChargeAfter_Helper::add_amount_failed_note( $order_id, $charge_amount );

				$order->update_status( 'failed', __( 'Failed payment', 'chargeafter-extension' ) );
			}
		} else {
			WC_ChargeAfter_Helper::add_general_failed_note( $order_id );
		}
	}

	/**
	 * Settle failed Indication
	 *
	 * @param $order_id
	 */
	private function settle_failed_indication( $order_id ) {
		WC_ChargeAfter_Helper::add_settle_failed_note( $order_id );

		$order = wc_get_order( $order_id );
		$order->update_status( 'failed', __( 'Failed payment', 'chargeafter-extension' ) );
	}

	/**
	 * Order hold to cancelled
	 *
	 * @param $order_id
	 */
	public function order_on_hold_to_cancelled( $order_id ) {
		if ( WC_ChargeAfter_Helper::is_chargeafter_gateway( $order_id ) ) {

			$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
			$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

			if ( $charge_data ) {
				switch ( $charge_data['state'] ) {
					case WC_ChargeAfter_ChargeState::AUTHORIZED:
						if ( WC_ChargeAfter_API::api_void_charge( $transaction_id ) ) {
							WC_ChargeAfter_Helper::add_void_auth_note( $order_id );
						} else {
							WC_ChargeAfter_Helper::add_general_failed_note( $order_id );
						}
						break;
				}
			}
		}
	}

	/**
	 * Order status change. Fixing the previous status
	 *
	 * @param $order_id
	 * @param $status_from
	 * @param $status_to
	 * @param $order WC_Order
	 */
	public function order_status_changed( $order_id, $status_from, $status_to, $order ) {
		if (
			WC_ChargeAfter_Helper::is_chargeafter_gateway( $order_id )
			&& ( 'on-hold' == $status_from && 'refunded' == $status_to )
		) {
			$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
			$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

			if ( $charge_data && in_array( $charge_data['state'], [ WC_ChargeAfter_ChargeState::AUTHORIZED ] ) ) {
				/*
				 * Last order note
				 */
				remove_filter( 'comments_clauses', array( 'WC_Comments', 'exclude_order_comments' ) );
				$notes = get_comments( array(
					'post_id' => $order_id,
					'orderby' => 'comment_ID',
					'order'   => 'DESC',
					'approve' => 'approve',
					'type'    => 'order_note',
					'number'  => 1
				) );
				add_filter( 'comments_clauses', array( 'WC_Comments', 'exclude_order_comments' ), 10, 1 );

				/*
				 * Delete order note
				 */
				wc_delete_order_note( $notes[0]->comment_ID );

				/*
				 * Set status
				 */
				$order->set_status( $status_from );
				$order->save();
			}
		}
	}

	/**
	 * Order hold to refunded
	 *
	 * @param $order_id
	 */
	public function order_status_refunded( $order_id ) {
		if ( WC_ChargeAfter_Helper::is_chargeafter_gateway( $order_id ) ) {
			$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
			$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

			$allow_refund_state = array(
				WC_ChargeAfter_ChargeState::SETTLED,
				WC_ChargeAfter_ChargeState::REFUNDED,
				WC_ChargeAfter_ChargeState::FULLY_REFUNDED,
				WC_ChargeAfter_ChargeState::PARTIALLY_REFUNDED,
				WC_ChargeAfter_ChargeState::PARTIALLY_SETTLED_REFUNDED,
				WC_ChargeAfter_ChargeState::SETTLED_PARTIALLY_REFUNDED,
				WC_ChargeAfter_ChargeState::PARTIALLY_SETTLED_PARTIALLY_REFUNDED,
			);

			if ( $charge_data && ! in_array( $charge_data['state'], $allow_refund_state ) ) {
				/*
				 * Prevent action
				 */
				remove_action( 'woocommerce_order_status_refunded', 'wc_order_fully_refunded' );

				/*
				 * Order note
				 */
				WC_ChargeAfter_Helper::add_refund_failed_note( $order_id );
			}
		}
	}

	/**
	 * Order complete | processing to cancelled
	 *
	 * @param $order_id
	 */
	public function order_processing_or_completed_to_cancelled( $order_id ) {
		if ( WC_ChargeAfter_Helper::is_chargeafter_gateway( $order_id ) ) {

			$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
			$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

			if ( $charge_data ) {
				switch ( $charge_data['state'] ) {
					case WC_ChargeAfter_ChargeState::PARTIALLY_REFUNDED:
					case WC_ChargeAfter_ChargeState::SETTLED_PARTIALLY_REFUNDED:
					case WC_ChargeAfter_ChargeState::PARTIALLY_SETTLED_PARTIALLY_REFUNDED:
					case WC_ChargeAfter_ChargeState::SETTLED:
						self::process_refund_amount( $order_id, $charge_data['totalAmount'] );
						break;
				}
			}
		}
	}

	/**
	 * Refund Amount
	 *
	 * @param $order_id
	 * @param $amount
	 *
	 * @return bool
	 */
	public static function process_refund_amount( $order_id, $amount ) {
		$refund_state   = false;
		$transaction_id = get_post_meta( $order_id, '_transaction_id', true );
		$charge_data    = WC_ChargeAfter_API::api_get_charge_data( $transaction_id );

		if ( $charge_data ) {
			switch ( $charge_data['state'] ) {
				case WC_ChargeAfter_ChargeState::PARTIALLY_REFUNDED:
				case WC_ChargeAfter_ChargeState::SETTLED_PARTIALLY_REFUNDED:
				case WC_ChargeAfter_ChargeState::PARTIALLY_SETTLED_PARTIALLY_REFUNDED:
				case WC_ChargeAfter_ChargeState::SETTLED:
					if (
						in_array( $charge_data['state'], array(
							WC_ChargeAfter_ChargeState::PARTIALLY_REFUNDED,
							WC_ChargeAfter_ChargeState::SETTLED_PARTIALLY_REFUNDED,
							WC_ChargeAfter_ChargeState::PARTIALLY_SETTLED_PARTIALLY_REFUNDED
						) )
						&& isset( $charge_data['refundedAmount'] ) && isset( $charge_data['totalAmount'] )
					) {
						$allow_refund_amount = ( $charge_data['totalAmount'] - $charge_data['refundedAmount'] );
						$amount              = ( $amount > $allow_refund_amount ? $allow_refund_amount : $amount );
					}

					$refund_data = WC_ChargeAfter_API::api_refund_amount( $transaction_id, $amount );

					if ( $refund_data && key_exists( 'id', $refund_data ) ) {
						WC_ChargeAfter_Helper::add_refund_note( $order_id, $amount, $refund_data['id'] );
						$refund_state = true;
					} else {
						WC_ChargeAfter_Helper::add_general_failed_note( $order_id );
					}

					break;

				default:
					$refund_state = new WP_Error( 'error', WC_ChargeAfter_Helper::REFUNDED_FAILED_NOTE );
					break;
			}
		}

		return $refund_state;
	}

    /**
     * Add custom fee to order
     *
     * @param $order_id
     * @param $amount
     *
     * @return void
     */
    private static function try_add_order_fee($order_id, $amount )
	{
		$order = wc_get_order( $order_id );

		if ( $order ) {
            $name = __( 'Price adjustment', 'chargeafter-extension' );
            $fee = self::get_order_fee_item( $name, $amount );

			$order->add_item( $fee );
			$order->calculate_totals( false );

			$order->save();

            WC_ChargeAfter_Helper::add_new_fee_note( $order_id, $name, $amount );
		}
	}

    /**
     * Generate fee item
     *
     * @param $name
     * @param $amount
     *
     * @return WC_Order_Item_Fee
     */
    private static function get_order_fee_item( $name, $amount )
    {
        $fee = new WC_Order_Item_Fee();

        $fee->set_name( $name );

        $fee->set_amount( $amount );
        $fee->set_total( $amount );

        $fee->set_total_tax( 0 );
		$fee->set_tax_status( 'none' );

		$fee->save();

        return $fee;
    }
}

/**
 * Init WC_ChargeAfter_Order_Handler
 */
new WC_ChargeAfter_Order_Handler();

