<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class WC_ChargeAfter_Start_Checkout_Script
 */
class WC_ChargeAfter_Start_Checkout_Script {
	/**
	 * Get Checkout Data
	 *
	 * @param $order WC_Order
	 *
	 * @return string
	 *
	 */
	public static function checkout_data( $order ) {
		/*
		 * Billing && Shipping Address
		 */
		$billing_address = self::address(
			$order->get_billing_address_1(), $order->get_billing_address_2(),
			$order->get_billing_city(), $order->get_billing_state(), $order->get_billing_postcode()
		);

		$shipping_address = self::address(
			$order->get_shipping_address_1(), $order->get_shipping_address_2(),
			$order->get_shipping_city(), $order->get_shipping_state(), $order->get_shipping_postcode()
		);

		$shipping_address_empty = empty(
			self::array_filter_recursive( $shipping_address, 'WC_ChargeAfter_Helper::filter_empty_string' )
		);

		$shipping_address = $shipping_address_empty ? $billing_address : $shipping_address;
		if (WC_ChargeAfter_Helper::is_enabled_data_update() && $shipping_address !== $billing_address) {
			return json_encode(
				[ 'error' => __( 'Billing and shipping address must be the same', 'chargeafter-extension' ) ]
			);
		}

		/*
		 * Items
		 */
		$items = array();
		if ( $order->get_items() ) {
			foreach ( $order->get_items() as $item_id => $item ) {
				$product_variation_id = $item['variation_id'];

				if ( $product_variation_id ) {
					$sku = get_post_meta( $item['variation_id'], '_sku', true );
				} else {
					$product = new WC_Product( $item['product_id'] );
					$sku     = $product->get_sku();
				}

				if ( ! $sku ) {
					$sku = $item_id;
				}

				$leasable = get_post_meta( $item['product_id'], 'wc_chargeafter_non-leasable', true) !== 'yes';

				$item_details = array(
					'name'     => strval( $item['name'] ),
					'price'    => floatval( $order->get_item_subtotal( $item ) ),
					'sku'      => strval( $sku ),
					'quantity' => intval( $item['qty'] ),
					'leasable' => boolval( $leasable ),
				);

				array_push( $items, $item_details );
			}
		}

		if ( $order->get_items( 'fee' ) ) {
			foreach ( $order->get_items( 'fee' ) as $item_id => $item_fee ) {
				$item_details = array(
					'name'     => strval( $item_fee->get_name() ),
					'price'    => floatval( $item_fee->get_total() ),
					'sku'      => strval( 'fee_' . $item_id ),
					'quantity' => 1,
					'leasable' => true,
				);

				array_push( $items, $item_details );
			}
		}

		/*
		 * Discounts
		 */
		$discounts = array();
		foreach ( $order->get_items( 'coupon' ) as $coupon_item_id => $coupon_item ) {
			$coupon_line = array(
				'name'   => strval( $coupon_item['name'] ),
				'amount' => floatval( $coupon_item['discount_amount'] )
			);

			array_push( $discounts, $coupon_line );
		}

		/*
		 * Checkout Data
		 */
		$checkout_data = array(
			'consumerDetails' => array(
				'firstName'          => strval( $order->get_billing_first_name() ),
				'lastName'           => strval( $order->get_billing_last_name() ),
				'email'              => strval( $order->get_billing_email() ),
				'mobilePhoneNumber'  => strval( $order->get_billing_phone() ),
				'merchantConsumerId' => strval( $order->get_user_id() ),
				'billingAddress'     => $billing_address,
				'shippingAddress'    => $shipping_address,
			),
			'cartDetails'     => array(
				'items'          => $items,
				'taxAmount'      => floatval( $order->get_total_tax() ),
				'shippingAmount' => floatval( $order->get_shipping_total() ),
				'totalAmount'    => floatval( $order->get_total() ),
			),
			'channel' => 'E_COMMERCE'
		);

		if ( !empty( get_locale() ) ) {
			$checkout_data['preferences'] = array(
				'language' => strval( get_locale() )
			);
		}

		if ( !empty( $order->get_currency() ) ) {
			$checkout_data['currency'] = strval( $order->get_currency() );
		}

		if ( count( $discounts ) > 0 ) {
			$checkout_data['cartDetails']['discounts'] = $discounts;
		}

		$filtered_data = self::array_filter_recursive( $checkout_data, 'WC_ChargeAfter_Helper::filter_empty_string' );

		return json_encode( $filtered_data );
	}

	/**
	 * Generate Address array
	 *
	 * @param $line1
	 * @param $line2
	 * @param $city
	 * @param $state
	 * @param $zip_code
	 *
	 * @return array
	 */
	public static function address( $line1, $line2, $city, $state, $zip_code ) {
		return array(
			'line1'   => strval( $line1 ),
			'line2'   => strval( $line2 ),
			'city'    => strval( $city ),
			'state'   => strval( $state ),
			'zipCode' => strval( $zip_code )
		);
	}

	/**
	 * Override array_filter_recursive
	 *
	 * @param $input
	 * @param null $callback
	 *
	 * @return array
	 */
	public static function array_filter_recursive( $input, $callback = null ) {
		foreach ( $input as &$value ) {
			if ( is_array( $value ) ) {
				$value = self::array_filter_recursive( $value, $callback );
			}
		}

		return array_filter( $input, $callback );
	}

	/**
	 * Start checkout script
	 *
	 * @param $public_key
	 * @param $url_suffix
	 *
	 * @return string
	 */
	public static function start_checkout_script( $public_key, $url_suffix ) {
		$onDataUpdate = '';
		if ( WC_ChargeAfter_Helper::is_enabled_data_update() ) {
			$onDataUpdate = trim("
				onDataUpdate: function(updatedData, callback) {
					if (callback === undefined && updatedData.callback) {
						callback = updatedData.callback;
					}
					
					data = updatedData;
					if (updatedData.updatedData) {
						data = updatedData.updatedData;
					}
					
					jQuery.post(
						'{{ON_DATA_UPDATE_URL}}&orderId=' + orderId + '&nonce=' + nonce,
						data,
						function(data) {
							callback(data);
						},
						'json',
					)
				},
			");
		}

		$template = "
	        <script>
				var caConfig = {
					apiKey: '{{PUBLIC_KEY}}'
				};
				
				!function(e,t,c,a,n){var r,o=t.getElementsByTagName(c)[0];
				e.ChargeAfter || (e.ChargeAfter = {}),
				t.getElementById(a)||(e.ChargeAfter.cfg = n,(r=t.createElement(c)).id=a,
				r.src='https://cdn{{URL_SUFFIX}}/web/v2/chargeafter.min.js?t='+1*new Date,
				r.async=!0,o.parentNode.insertBefore(r,o))}
				(window,document,'script','chargeafter-checkout-finance',caConfig);
	            
	            var presentCheckout = function(hash) {
	                var orderId = hash.match(/orderid=([a-zA-Z0-9]+)/)[1];
	                var nonce = hash.match(/nonce=([a-zA-Z0-9]+)/)[1];
	                
	                window.location.hash = '';
	                
	                jQuery.getJSON(
	                    '{{GET_CHECKOUT_DATA_URL}}&orderId=' + orderId + '&nonce=' + nonce,
	                    function(data) {
	                        if (data.error) {
	                            onError(data.error, 'CUSTOM_ERROR', orderId);
	                        } else {
								var opt = Object.assign(data, {
									{{ON_DATA_UPDATE_CALLBACK}}
									callback: function(token, data, error) {
										if (error) {
											onError(error.message, error.code, orderId)
										} else if (token) {
											jQuery.get(
												'{{CREATE_CHARGE_URL}}&orderId=' + orderId + '&confirmationToken=' + token + '&nonce=' + nonce
											)
											.done(function(redirect) {
												window.location.replace(redirect);
											})
											.fail(function(err) {
												window.location.replace(err.responseText);
											});
										}
									}
								});
								
								ChargeAfter.checkout.present(opt);
	                        }
	                    }
	                );
	            };
	            
	            var onError = function(message, code, orderId) {
	              jQuery.get(
						'{{GET_ERROR_URL}}&errorMsg=' + message + '&errorCode=' + code
					)
					.done(function(redirect) {
						jQuery.get(
							'{{GET_ERROR_CALLBACK_URL}}&orderId=' + orderId
						)
						.done()
						.fail()
						.always(function() {
							window.location.replace(redirect);
						});
					})
					.fail(
						window.location.reload()
					);
	            };
	             
	            window.addEventListener('hashchange', function() {
	                var hash = window.location.hash;
	                if (/#checkoutfinance=1/g.test(hash)) {
	                    presentCheckout(hash);
	                }
	            });
	         </script>
	    ";

		$home_url = home_url();

		$get_checkout_data_url = $home_url . '?wc-api=wc_chargeafter_get_checkout_data';
		$create_charge_url     = $home_url . '?wc-api=wc_chargeafter_create_charge';
		$create_on_update_url  = $home_url . '?wc-api=wc_chargeafter_on_data_update';
		$on_error_url          = $home_url . '?wc-api=wc_chargeafter_on_error_url';
		$on_error_callback_url = $home_url . '?wc-api=wc_chargeafter_on_error_callback_url';

		$template = str_replace( '{{ON_DATA_UPDATE_CALLBACK}}', $onDataUpdate, $template );
		$template = str_replace( '{{GET_CHECKOUT_DATA_URL}}', $get_checkout_data_url, $template );
		$template = str_replace( '{{CREATE_CHARGE_URL}}', $create_charge_url, $template );
		$template = str_replace( '{{ON_DATA_UPDATE_URL}}', $create_on_update_url, $template );
		$template = str_replace( '{{URL_SUFFIX}}', $url_suffix, $template );
		$template = str_replace( '{{GET_ERROR_CALLBACK_URL}}', $on_error_callback_url, $template );
		$template = str_replace( '{{GET_ERROR_URL}}', $on_error_url, $template );
		$template = str_replace( '{{PUBLIC_KEY}}', $public_key, $template );

		return preg_replace('!\s+!', ' ', $template);
	}
}
